/*
 * Ferry.cpp
 *
 *  Created on: 28.05.2009
 *      Author: ingmar
 */

#include "Ferry.h"

Ferry::Ferry(FerrySimulation& sim, odemx::random::ContinuousDist* crossing, unsigned int maxLoad, odemx::base::SimTime maxWaitTime):
	odemx::base::Process(sim, "Ferry"), crossing(crossing), maxLoad(maxLoad), maxWaitTime(maxWaitTime), ferrysim(&sim) {
	myTimer = new odemx::sync::Timer(sim, "FerryTimer");
	dryland = odemx::sync::PortHead::create(sim, "Dryland");
	island = odemx::sync::PortHead::create(sim, "Island");
}

Ferry::~Ferry() {
	delete myTimer;
}

int Ferry::main() {
	while (true) {
		//load at dryland
		while (carList.size() < maxLoad) {
			myTimer->setIn(maxWaitTime);
			odemx::sync::IMemory* who = wait(myTimer, dryland.get());
			if (who == myTimer)
				break;
			else if (who == dryland.get()) {
				myTimer->stop();
				std::auto_ptr<odemx::sync::PortData*> portData = dryland->get();
				Car* car = (Car*) *portData;
				holdFor(car->getLoadingTime());
				carList.push_back(car);
			}
		}

		//crossing the sea
		if (myTimer->isScheduled())
			myTimer->removeFromSchedule();
		holdFor(crossing->sample());

		//unload at island
		while (carList.size() != 0) {
			Car* car = carList.front();
			carList.pop_front();
			holdFor(car->getLoadingTime());
			throughput++;
			ferrysim->getMeanServiceTime()->update(car->calculateServiceTime(getSimulation().getTime()));
		}

		//load at island
		while (carList.size() < maxLoad) {
			myTimer->setIn(maxWaitTime);
			odemx::sync::IMemory* who = wait(myTimer, island.get());
			if (who == myTimer)
				break;
			else if (who == island.get()){
				myTimer->stop();
				std::auto_ptr<odemx::sync::PortData*> portData = island->get();
				Car* car = (Car*) *portData;
				holdFor(car->getLoadingTime());
				carList.push_back(car);
			}
		}

		//crossing the sea
		if (myTimer->isScheduled())
			myTimer->removeFromSchedule();
		holdFor(crossing->sample());

		//unload at island
		while (carList.size() != 0) {
			Car* car = carList.front();
			carList.pop_front();
			holdFor(car->getLoadingTime());
			throughput++;
			ferrysim->getMeanServiceTime()->update(car->calculateServiceTime(getSimulation().getTime()));
		}
	}
	return 0;
}

odemx::sync::PortTail::Ptr Ferry::getIslandTail() {
	return island->getTail();
}

odemx::sync::PortTail::Ptr Ferry::getDrylandTail() {
	return dryland->getTail();
}

long long Ferry::getThroughput() {
	return throughput;
}
