/*
 * FerrySimulation
 *
 *  Created on: 02.12.2009
 *      Author: ingmar
 */

#include "FerrySimulation.h"

FerrySimulation::FerrySimulation( double crossingTimeMean, double crossingTimeDeviation) :
	odemx::base::Simulation(), crossingTimeMean(crossingTimeMean), crossingTimeDeviation(crossingTimeDeviation) {
}

FerrySimulation::~FerrySimulation() {
	delete meanServiceTime;
}

void FerrySimulation::initSimulation() {

	meanServiceTime = new odemx::stats::Tally(*this, "mean service time");

	odemx::random::ContinuousDist * crossing = new odemx::random::Normal(*this, "crossing", crossingTimeMean, crossingTimeDeviation);
	odemx::random::ContinuousDist * loadingDryland = new odemx::random::Normal(*this, "loading dryland", 0.5, 0.2);
	odemx::random::ContinuousDist * loadingIsland = new odemx::random::Normal(*this, "loading island", 0.5, 0.2);
	
	odemx::random::ContinuousDist* dryland = new odemx::random::NegativeExponential(*this, "dryland", 0.1);
	odemx::random::ContinuousDist* island  = new odemx::random::NegativeExponential(*this, "island", 0.1);
	
	Ferry* ferry = new Ferry(*this, crossing, 4, 5.0);
	CarProducer* cpdl = new CarProducer(*this, "CarProducer dryland", ferry->getDrylandTail(), dryland, loadingDryland);
	CarProducer* cpil = new CarProducer(*this, "CarProducer island", ferry->getIslandTail(), island, loadingIsland);
	
	ferry->activate();
	cpdl->activate();
	cpil->activate();
		
}

odemx::stats::Tally* FerrySimulation::getMeanServiceTime() {
	return meanServiceTime;
}


